var parallelCoordsVizId = 'ParallelCoords';
var parallelCoordsPluginId = 'viz-'+parallelCoordsVizId;

pentaho = typeof pentaho == "undefined" ? {} : pentaho;

pentaho.viz = typeof pentaho.viz == "undefined" ? {} : pentaho.viz;

pentaho.visualizations = pentaho.visualizations || [];

pentaho.visualizations.push({
    id: parallelCoordsVizId,
    type: 'line',
    source: 'Protovis',
    name: 'Parallel Coords Chart',
    'class': 'pentaho.viz.ParallelCoords',
    needsColorGradient: false,
    customType: '',
    maxValues: [2000,2000,2000,5000],
    getDropZoneLabel: function(type){
      var label = type;
      return label;
    },
    args: {
    },
    propMap: [
    ],
    helper: {
        canRefreshReport: function(report) {
          var ok = true;
          if (report.findGemsByGembarId("cols").length == 0 && report.findGemsByGembarId("colorby").length == 0)
            ok = false;
          if (report.findGemsByGembarId("measures").length < 2)
            ok = false;
          return ok;
        }
    },
    dataReqs: [
      {
        name: 'Default',
        reqs :
            [
              {   id: 'cols', // Needed by Analyzer to indicate to render to column (and not rows)
                dataType: 'string',
                dataStructure: 'column',
                caption: 'items',
                required: false,
                allowMultiple: true
              },
              {   id: 'measures',
                dataType: 'number',
                dataStructure: 'column',
                caption: 'measures',
                required: true,
                allowMultiple: true
              },
              {
                  id: "optionsBtn",
                  dataType: 'none',
                  ui: {
                      group: "options",
                      type: "button",
                      label: "chartoptions"
                  }
              }
            ]
      }
    ]
  });

pen.require(["cdf/lib/CCC/protovis",
    "cdf/lib/CCC/protovis-msie"], function(pv){

pentaho.viz.ParallelCoords = function( div ) {

    this.debug = true;
    
    if( this.debug ) console.log('pentaho.viz.ParallelCoords.create');

    // Visualization API fields
    this.vizId = null; 
    this.controller = null;
    this.id = null;
    this.elementId = div.id;
    this.element = div

    this.data = [];
    this.dataCount = 0;
    this.labelColor = "#000000";
    this.w = 200;
    this.h = 200;    
    this.plotAreaColor = "rgba(240,240,240,";
    this.vis = null;
    this.backgroundColor = "rgba(255,255,255,";
    this.opacity = 0;
    this.rootPanel = null;
    this.paletteNo = 0;
    this.allStr = "All";
    this.sideMargin = 90;
    this.topMargin = 90;
    this.legendWidth = 150;
    this.paletteNo = 0;
    
    this.draw = function( dataTable, options ) {
        if( this.debug ) console.log('pentaho.viz.ParallelCoords.draw()');
        this.dataTable = dataTable;
        this.options = options;
        
        if( options ) {
            if( options.labelColor ) {
                this.labelColor = options.labelColor;
            }
            if( typeof options.opacity != 'undefined' ) {
                this.opacity = options.opacity;
            }
            if( options.backgroundColor ) {
                this.backgroundColor = options.backgroundColor;
            }
            if( options.sideMargin ) {
                this.sideMargin = options.sideMargin;
            }
            if( options.topMargin ) {
                this.topMargin = options.topMargin;
            }
        }
        
        // check the controller current visualizations arguments (set by the properties panel)
        if( this.controller && this.controller.currentViz && this.controller.currentViz.args ) {
            var labelColor = this.controller.currentViz.args.labelColor;
            if( labelColor ) { // TODO localize this?
                this.labelColor = labelColor;
            }

            if( typeof this.controller.currentViz.args.paletteno == 'string') {
                this.paletteNo = parseInt( this.controller.currentViz.args.paletteno );
            }

            if( typeof this.controller.currentViz.args.opacity == 'number' ) {
                this.opacity = this.controller.currentViz.args.opacity / 100;
            }
            if( typeof this.controller.currentViz.args.sidemargin == 'number' ) {
                this.sideMargin = this.controller.currentViz.args.sidemargin * 3;
                this.options.sideMargin = this.sideMargin;
            }
            if( typeof this.controller.currentViz.args.topmargin == 'number' ) {
                this.topMargin = this.controller.currentViz.args.topmargin * 2;
            }
            var labelFontFamily = this.controller.currentViz.args.labelFontFamily;
            if( labelFontFamily && labelFontFamily != 'Default') { // TODO localize this?
                this.labelFontFamily = labelFontFamily;
            }
            var labelSize = this.controller.currentViz.args.labelSize;
            if( labelSize ) {
                this.labelSize = labelSize;
            }
            var labelStyle = this.controller.currentViz.args.labelStyle;
            if( labelStyle && labelStyle != 'PLAIN') {
                this.labelStyle = labelStyle;
            }
            var backgroundType = this.controller.currentViz.args.backgroundType;
            if( backgroundType ) {
                this.backgroundType = backgroundType;
            }
            var backgroundColor = this.controller.currentViz.args.backgroundColor;
            if( backgroundColor ) {
                this.backgroundColor = backgroundColor;
            }
            var backgroundColorEnd = this.controller.currentViz.args.backgroundColorEnd;
            if( backgroundColorEnd ) {
                this.backgroundColorEnd = backgroundColorEnd;
            }
            
        }
        
        // check Analyzer's report document (yuck)
        if( cv && cv.rptEditor && cv.rptEditor.report && cv.rptEditor.report.reportDoc ) {
            var labelColor = cv.rptEditor.report.reportDoc.getChartOption('labelColor');
            if( labelColor ) {
                this.labelColor = labelColor;
                this.controller.currentViz.args.labelColor = labelColor;
            }
            var labelFontFamily = cv.rptEditor.report.reportDoc.getChartOption('labelFontFamily');
            if( labelFontFamily && labelFontFamily != 'Default') { // TODO localize this?
                this.labelFontFamily = labelFontFamily;
                this.controller.currentViz.args.labelFontFamily = labelFontFamily;
            }
            var labelSize = cv.rptEditor.report.reportDoc.getChartOption('labelSize');
            if( labelSize ) {
                this.labelSize = labelSize;
                this.controller.currentViz.args.labelSize = labelSize;
            }
            var labelStyle = cv.rptEditor.report.reportDoc.getChartOption('labelStyle');
            if( labelStyle && labelStyle != 'PLAIN') {
                this.labelStyle = labelStyle;
                this.controller.currentViz.args.labelStyle = labelStyle;
            }
            var backgroundType = cv.rptEditor.report.reportDoc.getChartOption('backgroundFill');
            if( backgroundType ) {
                this.backgroundType = backgroundType;
                this.controller.currentViz.args.backgroundType = backgroundType;
            }
            var backgroundColor = cv.rptEditor.report.reportDoc.getChartOption('backgroundColor');
            if( backgroundColor ) {
                this.backgroundColor = backgroundColor;
                this.controller.currentViz.args.backgroundColor = backgroundColor;
            }
            var backgroundColorEnd = cv.rptEditor.report.reportDoc.getChartOption('backgroundColorEnd');
            if( backgroundColorEnd ) {
                this.backgroundColorEnd = backgroundColorEnd;
                this.controller.currentViz.args.backgroundColorEnd = backgroundColorEnd;
            }
        }

        if( this.backgroundType == 'NONE' ) {
            this.element.style.background = 'none';
            this.element.style.backgroundColor = '';
        }
        else if( this.backgroundType == 'SOLID' ) {
            this.element.style.backgroundColor = this.backgroundColor;
        }
        else if( this.backgroundType == 'GRADIENT' ) {
            this.element.style.backgroundColor = '';            
            var origBack = this.element.style.background;
            if( div.style.background ) {
                var back = "-moz-linear-gradient(top, "+this.backgroundColor+", "+this.backgroundColorEnd+")";
                div.style.background = back;
                if( !div.style.background || div.style.background == origBack ) {
                    back = "-webkit-gradient(linear, left top, left bottom, from("+this.backgroundColor+"), to("+this.backgroundColorEnd+"))";
                    div.style.background = back;
                }
            }
            
        }
        
        this.labelFontStr = '';
        
        if( this.labelStyle && this.labelStyle != 'PLAIN') {
            this.labelFontStr += this.labelStyle + " ";
        }
        this.labelFontStr += this.labelSize+"pt ";
        if( this.labelFontFamily && this.labelFontFamily != 'Default') { // TODO localize this?
            this.labelFontStr += this.labelFontFamily;
        } else {
            this.labelFontStr += "Arial";
        }
        
        this.processData();
        
        this.resize();
    };
    
    
    
    this.resize = function() {
        
        if( this.debug ) console.log('pentaho.viz.ParallelCoords.resize()');

        /* Size parameters. */
        var padding = 30;
        var w = this.getClientWidth();
        var h = this.getClientHeight();

        this.dims = pv.keys(this.measures);
        this.fudge = 0.5;
        pentaho.viz.ParallelCoords.data = this.dataObj; 
        pentaho.viz.ParallelCoords.measures = this.measures; 

        for(id in this.measures ) {
            this.measures[id]['dataObj'] = this.dataObj;
        }

        if( this.debug ) console.log('creating scales and dictionaries');
        this.x = pv.Scale.ordinal(this.dims).splitFlush(0, w);
        this.dictFunc = dojo.hitch( this, function(t) { 
                return pv.Scale.linear(
                    this.dataObj.filter(function(d) { return !isNaN(d[t]); }),
                    function(d) { return Math.floor(d[t])-this.fudge; },
                    function(d) { return Math.ceil(d[t]) +this.fudge; }
                ).range(0, h);
            } );
        this.dictFunc2 = dojo.hitch( this, function(t) { 
                return pv.Scale.linear(
                    this.dataObj.filter(function(d) { return !isNaN(d[t]); }),
                    function(d) { return Math.floor(d[t])-this.fudge; },
                    function(d) { return Math.ceil(d[t]) +this.fudge; }
                ).range("steelblue", "brown");
            } );
            
        this.y = pv.dict(this.dims, this.dictFunc );
        
        this.c = pv.dict(this.dims, this.dictFunc2 );
        
        if( this.debug ) console.log('creating main panel');

        /* Interaction state. */
        this.filter = pv.dict(this.dims, dojo.hitch(this, function(t) {
        console.log( this.y[t].domain );
        console.log( this.y[t].domain() );
            var range = {min: this.y[t].domain()[0], max: this.y[t].domain()[1]};
console.log( 'range',range );
            return range;
          }));
          
        var active = this.dataTable.getColumnId(this.measuresCols[0]);

        /* The root panel. */
        var vis = new pv.Panel()
            .width(w-60)
            .height(h-40)
            .left(30)
            .right(30)
            .top(30)
            .bottom(20)
            .canvas(this.svgDiv);
    
        vis['measures'] = this.measures;
        vis['labelSize'] = this.labelSize;
        vis['colorFunc'] = this.getColor;
        vis['viz'] = this;
        vis['dims'] = this.dims;
        vis['data'] = this.dataObj;

        // The parallel coordinates display.
        if( this.debug ) console.log('creating parallel coordinates display');
        vis.add(pv.Panel)
            .data(this.dataObj)
            .visible( dojo.hitch( this, function(d) { 
                return this.dims.every( dojo.hitch( this, 
                    function(t) { 
                        return (d[t] >= this.filter[t].min) && (d[t] <= this.filter[t].max);
                    }
                ) );
            } 
            ) )
          .add(pv.Line)
            .data(this.dims)
            .left(function(t, d) { return this.root.viz.x(t); } )
            .bottom(function(t, d) { return this.root.viz.y[t](d[t]); } )
            .strokeStyle("#ddd")
            .lineWidth(1)
            .antialias(false);

        if( this.debug ) console.log('creating axes');
        // Rule per dimension.
        rule = vis.add(pv.Rule)
            .data(this.dims)
            .left(this.x);

        // Dimension label
        if( this.debug ) console.log('creating axes labels');
        rule.anchor("top").add(pv.Label)
            .top(-12)
            .font("bold 10px sans-serif")
            .text(function(d) { return this.root.viz.measures[d].name; });

        // The parallel coordinates display.
        if( this.debug ) console.log('creating lines');
        var change = vis.add(pv.Panel);

        var line = change.add(pv.Panel)
            .data(this.dataObj)
            .visible(function(d) { 
                return this.root.viz.dims.every( dojo.hitch( this, 
                    function(t) {
                        return (d[t] >= this.root.viz.filter[t].min) && (d[t] <= this.root.viz.filter[t].max);
                    }
                ) )
            })
          .add(pv.Line)
            .data(this.dims)
            .left(function(t, d) { return this.root.viz.x(t); } )
            .bottom(function(t, d) { return this.root.viz.y[t](d[t]); } )
            .strokeStyle(function(t, d) { return this.root.viz.c[active](d[active]); })
            .lineWidth(1);

        vis.render();

        return;

        /* Root panel. */
        

        if( this.debug ) console.log('creating cells');
        /* One cell per trait pair. */
        var cell = vis.add(pv.Panel)
            .data(this.measures)
            .top(function() { return (this.parent.topMargin) + this.index * (size + padding) + padding / 2; } )
            .height(size)
          .add(pv.Panel)
            .data(function(y) { return this.parent.parent.measures.map(function(x) { return ({px:x, py:y}); } ); } )
            .left(function() { return (this.parent.parent.sideMargin + this.parent.parent.legendWidth) + this.index * (size + padding) + padding / 2; } )
            .width(size);

        var legend = vis.add(pv.Panel)
            .width(this.legendWidth)
            .height(h)
            .left(0)
            .right(0);

        if( this.debug ) console.log('creating diagonal');


        /* Framed dot plots not along the diagonal. */
        var plot = cell.add(pv.Panel)
            .visible(function(t) { return t.px != t.py; } )
            .strokeStyle("#aaa");

        plot.sideMargin = this.sideMargin;

        /* X-axis ticks. */
        if( this.debug ) console.log('creating x-axis');
        var xtick = plot.add(pv.Rule)
            .data(function(t) { return position[t.px].ticks(5) } )
            .left(function(d, t) { return position[t.px](d) })
            .strokeStyle("#eee");

        /* Bottom label. */
        if( this.debug ) console.log('creating bottom label');
        xtick.anchor("bottom").add(pv.Label)
            .visible(function() { return (cell.parent.index == cell.parent.parent.measures.length - 1); } )
            .text(function(d, t) { return position[t.px].tickFormat(d); } )
            .textStyle(this.labelColor)
            .textAlign("right")
            .textAngle(-Math.PI/4)
            .font(this.labelFontStr);

        /* Top label. */
        if( this.debug ) console.log('creating top label');
        xtick.anchor("top").add(pv.Label)
            .visible(function() { return (cell.parent.index == 0); } )
            .text(function(d, t) { return position[t.px].tickFormat(d) } )
            .textStyle(this.labelColor)
            .textAlign("left")
            .textAngle(-Math.PI/4)
            .font(this.labelFontStr);

        /* Y-axis ticks. */
        if( this.debug ) console.log('creating y-axis ticks');
        var ytick = plot.add(pv.Rule)
            .data(function(t) { return position[t.py].ticks(5); } )
            .bottom(function(d, t) { return position[t.py](d); } )
            .strokeStyle("#eee");

        /* Left label. */
        if( this.debug ) console.log('creating left label');
        ytick.anchor("left").add(pv.Label)
            .visible(function() { return (cell.index == 0) } )
            .text(function(d, t) { return position[t.py].tickFormat(d); } )
            .textStyle(this.labelColor)
            .font(this.labelFontStr);

        /* Right label. */
        if( this.debug ) console.log('creating right label');
        ytick.anchor("right").add(pv.Label)
            .visible(function() { return (cell.index == cell.parent.parent.measures.length - 1); } )
            .text(function(d, t) { return position[t.py].tickFormat(d); } )
            .textStyle(this.labelColor)
            .font(this.labelFontStr);

        /* Frame and dot plot. */
        if( this.debug ) console.log('creating dot plots');
        window.dot = plot.add(pv.Dot)
            .data(this.dataObj)
            .left(function(d, t) { return position[t.px](d[t.px]); } )
            .bottom(function(d, t) { return position[t.py](d[t.py]); } )
            .shapeSize(10)
            .strokeStyle(null)
            .fillStyle(function(d) { return s
                && ((d[s.px] < s.x1) || (d[s.px] > s.x2)
                || (d[s.py] < s.y1) || (d[s.py] > s.y2))
                ? grey : this.parent.parent.parent.parent.colorFunc(d); } );

        /* Interaction: new selection and display and drag selection */
        if( this.debug ) console.log('creating selection panels');
        plot.add(pv.Panel)
           .data([{x:20, y:20, dx:100, dy:100}])
           .fillStyle("rgba(0,0,0,.05)")
           .cursor("crosshair")
           .events("all")
           .event("mousedown", pv.Behavior.select())
           .event("selectstart", function() { return (s = null, vis); } )
           .event("select", update )
         .add(pv.Bar)
           .visible(function(d, k, t) { return s && s.px == t.px && s.py == t.py; } )
           .left(function(d) { return d.x; } )
           .top(function(d) { return d.y; } )
           .width(function(d) { return d.dx; } )
           .height(function(d) { return d.dy; } )
           .fillStyle("rgba(0,0,0,.15)")
           .strokeStyle("white")
           .cursor("move")
           .event("mousedown", pv.Behavior.drag())
           .event("drag", update );

        /* Labels along the diagonal. */
        if( this.debug ) console.log('creating diagonal labels');
        cell.anchor("center").add(pv.Label)
            .visible(function(t) { return t.px == t.py; } )
            .textStyle(this.labelColor)
            .font(this.labelFontStr)
            .text(function(t) { return t.px.replace(/([WL])/, " $1").toLowerCase(); } );

        /* Legend. */
        if( this.debug ) console.log('creating legend');
        vis.add(pv.Dot)
            .data(this.colorBy)
            .top(function() { return 25 + this.index * (this.parent.labelSize * 1.8 ); } )
            .left( 10 )
            .shapeSize(20)
            .strokeStyle(null)
            .fillStyle(function(d) { return this.parent.colorFunc(d); })
            .font(this.labelFontStr)
          .anchor("right").add(pv.Label)
          .textStyle(this.labelColor)
            .font(this.labelFontStr);

        if( this.debug ) console.log('rendering');
        vis.render();

    }
    
    this.getColor = function( d ) {
    
    console.log(d);
        if( this.viz.paletteMap && d.colorByLabel ) {
            return this.viz.paletteMap[d.colorByLabel];
        } 
        else if( this.viz.paletteMap && this.viz.paletteMap[d] ) {
            return this.viz.paletteMap[d]
        } else {
            return pentaho.palettes[this.viz.paletteNo].colors[0];
        }
        return;
        
        var colors = pentaho.palettes[this.viz.paletteNo].colors;
        return colors[d % colors.length];
    };
    
    /* Compute new index values, rescale if needed, and render. */
    this.update = function() {
        if( this.debug ) console.log('pentaho.viz.ParallelCoords.update()');

    }
    
    this.init = function() {

        if( this.debug ) console.log('pentaho.viz.ParallelCoords.init()');
        this.allStr = pentaho.common.Messages.getString('all');

        this.svgDiv = document.createElement('DIV');
        this.element.appendChild( this.svgDiv );
        this.svgDiv.id = "svgDiv";
        this.svgDiv.style.position="absolute";
        this.svgDiv.style.top="0px";
        this.svgDiv.style.left="0px";
        this.svgDiv.style.width="100%";
        this.svgDiv.style.height="100%";   
        this.svgDiv.style.overflow="hidden";       
        this.svgDiv.style.textAlign="left";       
        if( this.debug ) console.log('pentaho.viz.ParallelCoords.done()');
    };
    
    this.processData = function() {
        if( this.debug ) console.log('pentaho.viz.ParallelCoords.processData()');

        this.measures = {
  cyl: {name: "cylinders", unit: ""},
  dsp: {name: "displacement", unit: " sq in"},
  lbs: {name: "weight", unit: " lbs"},
  hp: {name: "horsepower", unit: " hp"},
  acc: {name: "acceleration (0-60 mph)", unit: " sec"},
  mpg: {name: "mileage", unit: " mpg"},
  year: {name: "year", unit: ""}
}
    
        this.dataObj = [
{name:"chevrolet chevelle malibu", mpg:18, cyl:8, dsp:307, hp:130, lbs:3504, acc:12, year:70, origin:1},
{name:"buick skylark 320", mpg:15, cyl:8, dsp:350, hp:165, lbs:3693, acc:11.5, year:70, origin:1},
{name:"plymouth satellite", mpg:18, cyl:8, dsp:318, hp:150, lbs:3436, acc:11, year:70, origin:1},
{name:"amc rebel sst", mpg:16, cyl:8, dsp:304, hp:150, lbs:3433, acc:12, year:70, origin:1},
{name:"ford torino", mpg:17, cyl:8, dsp:302, hp:140, lbs:3449, acc:10.5, year:70, origin:1},
{name:"ford galaxie 500", mpg:15, cyl:8, dsp:429, hp:198, lbs:4341, acc:10, year:70, origin:1},
{name:"chevrolet impala", mpg:14, cyl:8, dsp:454, hp:220, lbs:4354, acc:9, year:70, origin:1},
{name:"plymouth fury iii", mpg:14, cyl:8, dsp:440, hp:215, lbs:4312, acc:8.5, year:70, origin:1},
{name:"pontiac catalina", mpg:14, cyl:8, dsp:455, hp:225, lbs:4425, acc:10, year:70, origin:1},
{name:"amc ambassador dpl", mpg:15, cyl:8, dsp:390, hp:190, lbs:3850, acc:8.5, year:70, origin:1},
{name:"citroen ds-21 pallas", mpg:undefined, cyl:4, dsp:133, hp:115, lbs:3090, acc:17.5, year:70, origin:2},
{name:"chevrolet chevelle concours (sw)", mpg:undefined, cyl:8, dsp:350, hp:165, lbs:4142, acc:11.5, year:70, origin:1},
{name:"ford torino (sw)", mpg:undefined, cyl:8, dsp:351, hp:153, lbs:4034, acc:11, year:70, origin:1},
{name:"plymouth satellite (sw)", mpg:undefined, cyl:8, dsp:383, hp:175, lbs:4166, acc:10.5, year:70, origin:1},
{name:"amc rebel sst (sw)", mpg:undefined, cyl:8, dsp:360, hp:175, lbs:3850, acc:11, year:70, origin:1},
{name:"dodge challenger se", mpg:15, cyl:8, dsp:383, hp:170, lbs:3563, acc:10, year:70, origin:1},
{name:"plymouth 'cuda 340", mpg:14, cyl:8, dsp:340, hp:160, lbs:3609, acc:8, year:70, origin:1},
{name:"ford mustang boss 302", mpg:undefined, cyl:8, dsp:302, hp:140, lbs:3353, acc:8, year:70, origin:1},
{name:"chevrolet monte carlo", mpg:15, cyl:8, dsp:400, hp:150, lbs:3761, acc:9.5, year:70, origin:1},
{name:"buick estate wagon (sw)", mpg:14, cyl:8, dsp:455, hp:225, lbs:3086, acc:10, year:70, origin:1},
{name:"toyota corona mark ii", mpg:24, cyl:4, dsp:113, hp:95, lbs:2372, acc:15, year:70, origin:3},
{name:"plymouth duster", mpg:22, cyl:6, dsp:198, hp:95, lbs:2833, acc:15.5, year:70, origin:1},
{name:"amc hornet", mpg:18, cyl:6, dsp:199, hp:97, lbs:2774, acc:15.5, year:70, origin:1},
{name:"ford maverick", mpg:21, cyl:6, dsp:200, hp:85, lbs:2587, acc:16, year:70, origin:1},
{name:"datsun pl510", mpg:27, cyl:4, dsp:97, hp:88, lbs:2130, acc:14.5, year:70, origin:3},
{name:"volkswagen 1131 deluxe sedan", mpg:26, cyl:4, dsp:97, hp:46, lbs:1835, acc:20.5, year:70, origin:2},
{name:"peugeot 504", mpg:25, cyl:4, dsp:110, hp:87, lbs:2672, acc:17.5, year:70, origin:2},
{name:"audi 100 ls", mpg:24, cyl:4, dsp:107, hp:90, lbs:2430, acc:14.5, year:70, origin:2},
{name:"saab 99e", mpg:25, cyl:4, dsp:104, hp:95, lbs:2375, acc:17.5, year:70, origin:2},
{name:"bmw 2002", mpg:26, cyl:4, dsp:121, hp:113, lbs:2234, acc:12.5, year:70, origin:2},
{name:"amc gremlin", mpg:21, cyl:6, dsp:199, hp:90, lbs:2648, acc:15, year:70, origin:1},
{name:"ford f250", mpg:10, cyl:8, dsp:360, hp:215, lbs:4615, acc:14, year:70, origin:1},
{name:"chevy c20", mpg:10, cyl:8, dsp:307, hp:200, lbs:4376, acc:15, year:70, origin:1},
{name:"dodge d200", mpg:11, cyl:8, dsp:318, hp:210, lbs:4382, acc:13.5, year:70, origin:1},
{name:"hi 1200d", mpg:9, cyl:8, dsp:304, hp:193, lbs:4732, acc:18.5, year:70, origin:1},
{name:"datsun pl510", mpg:27, cyl:4, dsp:97, hp:88, lbs:2130, acc:14.5, year:71, origin:3},
{name:"chevrolet vega 2300", mpg:28, cyl:4, dsp:140, hp:90, lbs:2264, acc:15.5, year:71, origin:1},
{name:"toyota corona", mpg:25, cyl:4, dsp:113, hp:95, lbs:2228, acc:14, year:71, origin:3},
{name:"ford pinto", mpg:25, cyl:4, dsp:98, hp:undefined, lbs:2046, acc:19, year:71, origin:1},
{name:"volkswagen super beetle 117", mpg:undefined, cyl:4, dsp:97, hp:48, lbs:1978, acc:20, year:71, origin:2},
{name:"amc gremlin", mpg:19, cyl:6, dsp:232, hp:100, lbs:2634, acc:13, year:71, origin:1},
{name:"plymouth satellite custom", mpg:16, cyl:6, dsp:225, hp:105, lbs:3439, acc:15.5, year:71, origin:1},
{name:"chevrolet chevelle malibu", mpg:17, cyl:6, dsp:250, hp:100, lbs:3329, acc:15.5, year:71, origin:1},
{name:"ford torino 500", mpg:19, cyl:6, dsp:250, hp:88, lbs:3302, acc:15.5, year:71, origin:1},
{name:"amc matador", mpg:18, cyl:6, dsp:232, hp:100, lbs:3288, acc:15.5, year:71, origin:1},
{name:"chevrolet impala", mpg:14, cyl:8, dsp:350, hp:165, lbs:4209, acc:12, year:71, origin:1},
{name:"pontiac catalina brougham", mpg:14, cyl:8, dsp:400, hp:175, lbs:4464, acc:11.5, year:71, origin:1},
{name:"ford galaxie 500", mpg:14, cyl:8, dsp:351, hp:153, lbs:4154, acc:13.5, year:71, origin:1},
{name:"plymouth fury iii", mpg:14, cyl:8, dsp:318, hp:150, lbs:4096, acc:13, year:71, origin:1},
{name:"dodge monaco (sw)", mpg:12, cyl:8, dsp:383, hp:180, lbs:4955, acc:11.5, year:71, origin:1},
{name:"ford country squire (sw)", mpg:13, cyl:8, dsp:400, hp:170, lbs:4746, acc:12, year:71, origin:1}
];


        this.rowsCols = [];
        this.measuresCols = [];
//        this.measures = [];
        
        for( var colNo=0; colNo<this.dataTable.getNumberOfColumns(); colNo++) {
            var dataReq = this.dataTable.getColumnProperty(colNo,'dataReq');
            if(dataReq) {
                for (var idx=0; idx < dataReq.length; idx++) {
                    if( dataReq[idx].id == 'cols' ) {
                        this.rowsCols.push( colNo );
                    }
                    else if( dataReq[idx].id == 'measures' ) {
                        this.measuresCols.push( colNo );
//                        this.measures.push( this.dataTable.getColumnLabel( colNo ) );
                    }
                }
            }
        }
        /*
        if( this.colorByCol != -1 ) {
            this.colorBy = this.dataTable.getDistinctFormattedValues(this.colorByCol)
            var metrics = this.options.metrics[this.colorByCol];
            this.paletteMap = metrics.paletteMap;
        } else {
            this.colorBy = [this.allStr];
        }
        this.dataObj = [];
        // create the data object
        for( var rowNo=0; rowNo<this.dataTable.getNumberOfRows(); rowNo++ ) {
            var obj = {
            };
            if( this.colorByCol != -1 ) {
                obj.colorBy = this.dataTable.getValue( rowNo, this.colorByCol);
                obj.colorByLabel = this.dataTable.getFormattedValue( rowNo, this.colorByCol);
            } else {
                obj.colorBy = this.allStr;
                obj.colorByLabel = this.allStr;
            }
            for( var idx=0; idx<this.measuresCols.length; idx++ ) {
                obj[this.measures[idx]] = this.dataTable.getValue( rowNo, this.measuresCols[idx] );
            }
            this.dataObj.push( obj );
        }
*/
    }
    
    this.getClientWidth = function() {
      return this.element.offsetWidth;
    };

    this.getClientHeight = function() {
      return this.element.offsetHeight;
    };
    
    this.init();
    
}

} );

    /* Interaction: update selection. */
    function update(d, t) {
        console.log( 'this.sideMargin', this.parent.sideMargin );
      s = d;
      s.px = t.px;
      s.py = t.py;
      s.x1 = position[t.px].invert(d.x);
      s.x2 = position[t.px].invert(d.x + d.dx);
      s.y1 = position[t.py].invert(size - d.y - d.dy);
      s.y2 = position[t.py].invert(size - d.y);
      dot.context(null, 0, function() { this.render() });
    }
    
// globals - yuck 
pentaho.viz.ParallelCoords.data; 
pentaho.viz.ParallelCoords.measures; 
          
    
